@@ -21,13 +21,13 @@ import os
 import kp
 import sphinx_material
 
-project = 'Vulkan Kompute'
+project = 'Kompute'
 copyright = '2020, The Institute for Ethical AI & Machine Learning'
-html_title = "Vulkan Kompute Documentation (Python & C++)"
+html_title = "Kompute Documentation (Python & C++)"
 author = 'Alejandro Saucedo'
 
 # The full version, including alpha/beta/rc tags
-release = '0.7.0'
+release = '0.8.0'
 
 
 # -- General configuration ---------------------------------------------------
@@ -73,15 +73,15 @@ if html_theme == 'sphinx_material':
         'base_url': 'https://kompute.cc',
         'color_primary': 'red',
         'color_accent': 'light-blue',
-        'repo_url': 'https://github.com/EthicalML/vulkan-kompute/',
-        'repo_name': 'Vulkan Kompute',
+        'repo_url': 'https://github.com/KomputeProject/kompute/',
+        'repo_name': 'Kompute',
         'globaltoc_depth': 2,
         'globaltoc_collapse': False,
         'globaltoc_includehidden': False,
         "repo_type": "github",
         "nav_links": [
             {
-                "href": "https://github.com/EthicalML/vulkan-kompute/",
+                "href": "https://github.com/KomputeProject/kompute/",
                 "internal": False,
                 "title": "Kompute Repo",
             },
