@@ -6,14 +6,39 @@
 
 #include "fmt/ranges.h"
 
+#include "utils.hpp"
 #include "docstrings.hpp"
 
 namespace py = pybind11;
-using namespace pybind11::literals; // for the `_a` literal
 
 //used in Core.hpp
 py::object kp_debug, kp_info, kp_warning, kp_error;
 
+std::unique_ptr<kp::OpAlgoDispatch> opAlgoDispatchPyInit(
+                        std::shared_ptr<kp::Algorithm>& algorithm,
+                        const py::array& push_consts) {
+    const py::buffer_info info        = push_consts.request();
+    KP_LOG_DEBUG("Kompute Python Manager creating tensor_T with push_consts size {} dtype {}",
+            push_consts.size(), std::string(py::str(push_consts.dtype())));
+
+
+    if (push_consts.dtype() == py::dtype::of<std::float_t>()) {
+        std::vector<float> dataVec((float*)info.ptr, ((float*)info.ptr) + info.size);
+        return std::unique_ptr<kp::OpAlgoDispatch>{new kp::OpAlgoDispatch(algorithm, dataVec)};
+    } else if (push_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+        std::vector<uint32_t> dataVec((uint32_t*)info.ptr, ((uint32_t*)info.ptr) + info.size);
+        return std::unique_ptr<kp::OpAlgoDispatch>{new kp::OpAlgoDispatch(algorithm, dataVec)};
+    } else if (push_consts.dtype() == py::dtype::of<std::int32_t>()) {
+        std::vector<int32_t> dataVec((int32_t*)info.ptr, ((int32_t*)info.ptr) + info.size);
+        return std::unique_ptr<kp::OpAlgoDispatch>{new kp::OpAlgoDispatch(algorithm, dataVec)};
+    } else if (push_consts.dtype() == py::dtype::of<std::double_t>()) {
+        std::vector<double> dataVec((double*)info.ptr, ((double*)info.ptr) + info.size);
+        return std::unique_ptr<kp::OpAlgoDispatch>{new kp::OpAlgoDispatch(algorithm, dataVec)};
+    } else {
+        throw std::runtime_error("Kompute Python no valid dtype supported");
+    }
+}
+
 PYBIND11_MODULE(kp, m) {
 
     // The logging modules are used in the Kompute.hpp file
@@ -33,34 +58,6 @@ PYBIND11_MODULE(kp, m) {
         .value("storage", kp::Tensor::TensorTypes::eStorage, DOC(kp, Tensor, TensorTypes, eStorage))
         .export_values();
 
-#if !defined(KOMPUTE_DISABLE_SHADER_UTILS) || !KOMPUTE_DISABLE_SHADER_UTILS
-    py::class_<kp::Shader>(m, "Shader", "Shader class")
-        .def_static("compile_source", [](
-                                    const std::string& source,
-                                    const std::string& entryPoint,
-                                    const std::vector<std::pair<std::string,std::string>>& definitions) {
-                std::vector<uint32_t> spirv = kp::Shader::compileSource(source, entryPoint, definitions);
-                return py::bytes((const char*)spirv.data(), spirv.size() * sizeof(uint32_t));
-            },
-            DOC(kp, Shader, compileSource),
-            py::arg("source"),
-            py::arg("entryPoint") = "main",
-            py::arg("definitions") = std::vector<std::pair<std::string,std::string>>() )
-        .def_static("compile_sources", [](
-                                    const std::vector<std::string>& source,
-                                    const std::vector<std::string>& files,
-                                    const std::string& entryPoint,
-                                    const std::vector<std::pair<std::string,std::string>>& definitions) {
-                std::vector<uint32_t> spirv = kp::Shader::compileSources(source, files, entryPoint, definitions);
-                return py::bytes((const char*)spirv.data(), spirv.size() * sizeof(uint32_t));
-            },
-            DOC(kp, Shader, compileSources),
-            py::arg("sources"),
-            py::arg("files") = std::vector<std::string>(),
-            py::arg("entryPoint") = "main",
-            py::arg("definitions") = std::vector<std::pair<std::string,std::string>>() );
-#endif // KOMPUTE_DISABLE_SHADER_UTILS
-
     py::class_<kp::OpBase, std::shared_ptr<kp::OpBase>>(m, "OpBase", DOC(kp, OpBase));
 
     py::class_<kp::OpTensorSyncDevice, std::shared_ptr<kp::OpTensorSyncDevice>>(
@@ -77,9 +74,12 @@ PYBIND11_MODULE(kp, m) {
 
     py::class_<kp::OpAlgoDispatch, std::shared_ptr<kp::OpAlgoDispatch>>(
             m, "OpAlgoDispatch", py::base<kp::OpBase>(), DOC(kp, OpAlgoDispatch))
-        .def(py::init<const std::shared_ptr<kp::Algorithm>&,const kp::Constants&>(),
+        .def(py::init<const std::shared_ptr<kp::Algorithm>&,const std::vector<float>&>(),
                 DOC(kp, OpAlgoDispatch, OpAlgoDispatch),
-                py::arg("algorithm"), py::arg("push_consts") = kp::Constants());
+                py::arg("algorithm"), py::arg("push_consts") = std::vector<float>())
+        .def(py::init(&opAlgoDispatchPyInit),
+                DOC(kp, OpAlgoDispatch, OpAlgoDispatch),
+                py::arg("algorithm"), py::arg("push_consts"));
 
     py::class_<kp::OpMult, std::shared_ptr<kp::OpMult>>(
             m, "OpMult", py::base<kp::OpBase>(), DOC(kp, OpMult))
@@ -89,24 +89,22 @@ PYBIND11_MODULE(kp, m) {
     py::class_<kp::Algorithm, std::shared_ptr<kp::Algorithm>>(m, "Algorithm", DOC(kp, Algorithm, Algorithm))
         .def("get_tensors", &kp::Algorithm::getTensors, DOC(kp, Algorithm, getTensors))
         .def("destroy", &kp::Algorithm::destroy, DOC(kp, Algorithm, destroy))
-        .def("get_spec_consts", &kp::Algorithm::getSpecializationConstants, DOC(kp, Algorithm, getSpecializationConstants))
         .def("is_init", &kp::Algorithm::isInit, DOC(kp, Algorithm, isInit));
 
     py::class_<kp::Tensor, std::shared_ptr<kp::Tensor>>(m, "Tensor", DOC(kp, Tensor))
         .def("data", [](kp::Tensor& self) {
                 // Non-owning container exposing the underlying pointer
-                py::str dummyDataOwner; // Explicitly request data to not be owned by np
                 switch (self.dataType()) {
                 case kp::Tensor::TensorDataTypes::eFloat:
-                    return py::array(self.size(), self.data<float>(), dummyDataOwner);
+                    return py::array(self.size(), self.data<float>(), py::cast(&self));
                 case kp::Tensor::TensorDataTypes::eUnsignedInt:
-                    return py::array(self.size(), self.data<uint32_t>(), dummyDataOwner);
+                    return py::array(self.size(), self.data<uint32_t>(), py::cast(&self));
                 case kp::Tensor::TensorDataTypes::eInt:
-                    return py::array(self.size(), self.data<int32_t>(), dummyDataOwner);
+                    return py::array(self.size(), self.data<int32_t>(), py::cast(&self));
                 case kp::Tensor::TensorDataTypes::eDouble:
-                    return py::array(self.size(), self.data<double>(), dummyDataOwner);
+                    return py::array(self.size(), self.data<double>(), py::cast(&self));
                 case kp::Tensor::TensorDataTypes::eBool:
-                    return py::array(self.size(), self.data<bool>(), dummyDataOwner);
+                    return py::array(self.size(), self.data<bool>(), py::cast(&self));
                 default:
                     throw std::runtime_error("Kompute Python data type not supported");
                 }
@@ -156,6 +154,8 @@ PYBIND11_MODULE(kp, m) {
                 py::arg("device") = 0,
                 py::arg("family_queue_indices") = std::vector<uint32_t>(),
                 py::arg("desired_extensions") = std::vector<std::string>())
+        .def("destroy", &kp::Manager::destroy,
+                DOC(kp, Manager, destroy))
         .def("sequence", &kp::Manager::sequence, DOC(kp, Manager, sequence),
                 py::arg("queue_index") = 0, py::arg("total_timestamps") = 0)
         .def("tensor", [np](kp::Manager& self,
@@ -206,8 +206,8 @@ PYBIND11_MODULE(kp, m) {
                              const std::vector<std::shared_ptr<kp::Tensor>>& tensors,
                              const py::bytes& spirv,
                              const kp::Workgroup& workgroup,
-                             const kp::Constants& spec_consts,
-                             const kp::Constants& push_consts) {
+                             const std::vector<float>& spec_consts,
+                             const std::vector<float>& push_consts) {
                     py::buffer_info info(py::buffer(spirv).request());
                     const char *data = reinterpret_cast<const char *>(info.ptr);
                     size_t length = static_cast<size_t>(info.size);
@@ -218,25 +218,121 @@ PYBIND11_MODULE(kp, m) {
             py::arg("tensors"),
             py::arg("spirv"),
             py::arg("workgroup") = kp::Workgroup(),
-            py::arg("spec_consts") = kp::Constants(),
-            py::arg("push_consts") = kp::Constants())
-        .def("get_device_properties", [](kp::Manager& self){
-            const auto properties = self.getDeviceProperties();
-            py::dict py_props(
-                "device_name"_a = std::string(properties.deviceName.data()),
-                "max_work_group_count"_a       = py::make_tuple(properties.limits.maxComputeWorkGroupCount[0],
-                                                                properties.limits.maxComputeWorkGroupCount[1],
-                                                                properties.limits.maxComputeWorkGroupCount[2]),
-                "max_work_group_invocations"_a = properties.limits.maxComputeWorkGroupInvocations,
-                "max_work_group_size"_a        = py::make_tuple(properties.limits.maxComputeWorkGroupSize[0],
-                                                                properties.limits.maxComputeWorkGroupSize[1],
-                                                                properties.limits.maxComputeWorkGroupSize[2]),
-                "timestamps_supported"_a       = (bool)properties.limits.timestampComputeAndGraphics
-            );
+            py::arg("spec_consts") = std::vector<float>(),
+            py::arg("push_consts") = std::vector<float>())
+        .def("algorithm", [np](kp::Manager& self,
+                             const std::vector<std::shared_ptr<kp::Tensor>>& tensors,
+                             const py::bytes& spirv,
+                             const kp::Workgroup& workgroup,
+                             const py::array& spec_consts,
+                             const py::array& push_consts) {
 
-            return py_props;
+                py::buffer_info info(py::buffer(spirv).request());
+                const char *data = reinterpret_cast<const char *>(info.ptr);
+                size_t length = static_cast<size_t>(info.size);
+                std::vector<uint32_t> spirvVec((uint32_t*)data, (uint32_t*)(data + length));
+
+                const py::buffer_info pushInfo        = push_consts.request();
+                const py::buffer_info specInfo        = spec_consts.request();
+
+                KP_LOG_DEBUG("Kompute Python Manager creating Algorithm_T with "
+                        "push consts data size {} dtype {} and spec const data size {} dtype {}",
+                        push_consts.size(), std::string(py::str(push_consts.dtype())),
+                        spec_consts.size(), std::string(py::str(spec_consts.dtype())));
+
+                // We have to iterate across a combination of parameters due to the lack of support for templating
+                if (spec_consts.dtype() == py::dtype::of<std::float_t>()) {
+                    std::vector<float> specConstsVec((float*)specInfo.ptr, ((float*)specInfo.ptr) + specInfo.size);
+                    if (spec_consts.dtype() == py::dtype::of<std::float_t>()) {
+                        std::vector<float> pushConstsVec((float*)pushInfo.ptr, ((float*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specConstsVec, pushConstsVec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::int32_t>()) {
+                        std::vector<int32_t> pushConstsVec((int32_t*)pushInfo.ptr, ((int32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specConstsVec, pushConstsVec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+                        std::vector<uint32_t> pushConstsVec((uint32_t*)pushInfo.ptr, ((uint32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specConstsVec, pushConstsVec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::double_t>()) {
+                        std::vector<double> pushConstsVec((double*)pushInfo.ptr, ((double*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specConstsVec, pushConstsVec);
+                    }
+                } else if (spec_consts.dtype() == py::dtype::of<std::int32_t>()) {
+                    std::vector<int32_t> specconstsvec((int32_t*)specInfo.ptr, ((int32_t*)specInfo.ptr) + specInfo.size);
+                    if (spec_consts.dtype() == py::dtype::of<std::float_t>()) {
+                        std::vector<float> pushconstsvec((float*)pushInfo.ptr, ((float*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::int32_t>()) {
+                        std::vector<int32_t> pushconstsvec((int32_t*)pushInfo.ptr, ((int32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+                        std::vector<uint32_t> pushconstsvec((uint32_t*)pushInfo.ptr, ((uint32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::double_t>()) {
+                        std::vector<double> pushconstsvec((double*)pushInfo.ptr, ((double*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    }
+                } else if (spec_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+                    std::vector<uint32_t> specconstsvec((uint32_t*)specInfo.ptr, ((uint32_t*)specInfo.ptr) + specInfo.size);
+                    if (spec_consts.dtype() == py::dtype::of<std::float_t>()) {
+                        std::vector<float> pushconstsvec((float*)pushInfo.ptr, ((float*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::int32_t>()) {
+                        std::vector<int32_t> pushconstsvec((int32_t*)pushInfo.ptr, ((int32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+                        std::vector<uint32_t> pushconstsvec((uint32_t*)pushInfo.ptr, ((uint32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::double_t>()) {
+                        std::vector<double> pushconstsvec((double*)pushInfo.ptr, ((double*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    }
+                } else if (spec_consts.dtype() == py::dtype::of<std::double_t>()) {
+                    std::vector<double> specconstsvec((double*)specInfo.ptr, ((double*)specInfo.ptr) + specInfo.size);
+                    if (spec_consts.dtype() == py::dtype::of<std::float_t>()) {
+                        std::vector<float> pushconstsvec((float*)pushInfo.ptr, ((float*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::int32_t>()) {
+                        std::vector<float> pushconstsvec((int32_t*)pushInfo.ptr, ((int32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::uint32_t>()) {
+                        std::vector<float> pushconstsvec((uint32_t*)pushInfo.ptr, ((uint32_t*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    } else if (spec_consts.dtype() == py::dtype::of<std::double_t>()) {
+                        std::vector<float> pushconstsvec((double*)pushInfo.ptr, ((double*)pushInfo.ptr) + pushInfo.size);
+                        return self.algorithm(tensors, spirvVec, workgroup, specconstsvec, pushconstsvec);
+                    }
+                } else {
+                    // If reach then no valid dtype supported
+                    throw std::runtime_error("Kompute Python no valid dtype supported");
+                }
+            },
+            DOC(kp, Manager, algorithm),
+            py::arg("tensors"),
+            py::arg("spirv"),
+            py::arg("workgroup") = kp::Workgroup(),
+            py::arg("spec_consts") = std::vector<float>(),
+            py::arg("push_consts") = std::vector<float>())
+        .def("list_devices", [](kp::Manager& self){
+            const std::vector<vk::PhysicalDevice> devices = self.listDevices();
+            py::list list;
+            for (const vk::PhysicalDevice& device : devices) {
+                list.append(kp::py::vkPropertiesToDict(device.getProperties()));
+            }
+            return list;
+        }, "Return a dict containing information about the device")
+        .def("get_device_properties", [](kp::Manager& self){
+            const vk::PhysicalDeviceProperties properties = self.getDeviceProperties();
+
+            return kp::py::vkPropertiesToDict(properties);
         }, "Return a dict containing information about the device");
 
+    auto atexit = py::module_::import("atexit");
+    atexit.attr("register")(py::cpp_function([](){
+        kp_debug = py::none();
+        kp_info = py::none();
+        kp_warning = py::none();
+        kp_error = py::none();
+    }));
 
 #ifdef VERSION_INFO
     m.attr("__version__") = VERSION_INFO;
@@ -244,3 +340,4 @@ PYBIND11_MODULE(kp, m) {
     m.attr("__version__") = "dev";
 #endif
 }
+
