@@ -1,8 +1,11 @@
+// SPDX-License-Identifier: Apache-2.0
 
 #include "gtest/gtest.h"
 
 #include "kompute/Kompute.hpp"
 
+#include "kompute_test/Shader.hpp"
+
 TEST(TestMultipleAlgoExecutions, TestEndToEndFunctionality)
 {
 
@@ -46,12 +49,12 @@ TEST(TestMultipleAlgoExecutions, TestEndToEndFunctionality)
     };
 
     kp::Workgroup workgroup({ 3, 1, 1 });
-    kp::Constants specConsts({ 2 });
-    kp::Constants pushConstsA({ 2.0 });
-    kp::Constants pushConstsB({ 3.0 });
+    std::vector<float> specConsts({ 2 });
+    std::vector<float> pushConstsA({ 2.0 });
+    std::vector<float> pushConstsB({ 3.0 });
 
     auto algorithm = mgr.algorithm(params,
-                                   kp::Shader::compileSource(shader),
+                                   compileSource(shader),
                                    workgroup,
                                    specConsts,
                                    pushConstsA);
@@ -89,13 +92,24 @@ TEST(TestMultipleAlgoExecutions, SingleSequenceRecord)
           pa[index] = pa[index] + 1;
       })");
 
-    std::vector<uint32_t> spirv = kp::Shader::compileSource(shader);
+    std::vector<uint32_t> spirv = compileSource(shader);
 
     {
+        // A sharedMemoryBarrier is required as the shader is not thread-safe:w
+        std::shared_ptr<kp::OpMemoryBarrier> shaderBarrier{
+                  new kp::OpMemoryBarrier({ tensorA },
+                  vk::AccessFlagBits::eTransferRead,
+                  vk::AccessFlagBits::eShaderWrite,
+                  vk::PipelineStageFlagBits::eComputeShader,
+                  vk::PipelineStageFlagBits::eComputeShader)
+        };
+
         mgr.sequence()
           ->record<kp::OpTensorSyncDevice>({ tensorA })
           ->record<kp::OpAlgoDispatch>(mgr.algorithm({ tensorA }, spirv))
+          ->record(shaderBarrier)
           ->record<kp::OpAlgoDispatch>(mgr.algorithm({ tensorA }, spirv))
+          ->record(shaderBarrier)
           ->record<kp::OpAlgoDispatch>(mgr.algorithm({ tensorA }, spirv))
           ->record<kp::OpTensorSyncLocal>({ tensorA })
           ->eval();
@@ -119,7 +133,7 @@ TEST(TestMultipleAlgoExecutions, MultipleCmdBufRecords)
           pa[index] = pa[index] + 1;
       })");
 
-    std::vector<uint32_t> spirv = kp::Shader::compileSource(shader);
+    std::vector<uint32_t> spirv = compileSource(shader);
 
     std::shared_ptr<kp::Algorithm> algorithm =
       mgr.algorithm({ tensorA }, spirv);
@@ -155,7 +169,7 @@ TEST(TestMultipleAlgoExecutions, MultipleSequences)
           pa[index] = pa[index] + 1;
       })");
 
-    std::vector<uint32_t> spirv = kp::Shader::compileSource(shader);
+    std::vector<uint32_t> spirv = compileSource(shader);
 
     std::shared_ptr<kp::Algorithm> algorithm =
       mgr.algorithm({ tensorA }, spirv);
@@ -190,7 +204,7 @@ TEST(TestMultipleAlgoExecutions, SingleRecordMultipleEval)
           pa[index] = pa[index] + 1;
       })");
 
-    std::vector<uint32_t> spirv = kp::Shader::compileSource(shader);
+    std::vector<uint32_t> spirv = compileSource(shader);
 
     std::shared_ptr<kp::Algorithm> algorithm =
       mgr.algorithm({ tensorA }, spirv);
@@ -205,3 +219,60 @@ TEST(TestMultipleAlgoExecutions, SingleRecordMultipleEval)
 
     EXPECT_EQ(tensorA->vector(), std::vector<float>({ 3, 3, 3 }));
 }
+
+TEST(TestMultipleAlgoExecutions, TestAlgorithmUtilFunctions)
+{
+
+    kp::Manager mgr;
+
+    // Default tensor constructor simplifies creation of float values
+    auto tensorInA = mgr.tensor({ 2., 2., 2. });
+    auto tensorInB = mgr.tensor({ 1., 2., 3. });
+    // Explicit type constructor supports int, in32, double, float and int
+    auto tensorOutA = mgr.tensorT<uint32_t>({ 0, 0, 0 });
+    auto tensorOutB = mgr.tensorT<uint32_t>({ 0, 0, 0 });
+
+    std::string shader = (R"(
+        #version 450
+
+        layout (local_size_x = 1) in;
+
+        // The input tensors bind index is relative to index in parameter passed
+        layout(set = 0, binding = 0) buffer buf_in_a { float in_a[]; };
+        layout(set = 0, binding = 1) buffer buf_in_b { float in_b[]; };
+        layout(set = 0, binding = 2) buffer buf_out_a { uint out_a[]; };
+        layout(set = 0, binding = 3) buffer buf_out_b { uint out_b[]; };
+
+        // Kompute supports push constants updated on dispatch
+        layout(push_constant) uniform PushConstants {
+            float val;
+        } push_const;
+
+        // Kompute also supports spec constants on initalization
+        layout(constant_id = 0) const float const_one = 0;
+
+        void main() {
+            uint index = gl_GlobalInvocationID.x;
+            out_a[index] += uint( in_a[index] * in_b[index] );
+            out_b[index] += uint( const_one * push_const.val );
+        }
+    )");
+
+    std::vector<std::shared_ptr<kp::Tensor>> params = {
+        tensorInA, tensorInB, tensorOutA, tensorOutB
+    };
+
+    kp::Workgroup workgroup({ 3, 1, 1 });
+    std::vector<float> specConsts({ 2 });
+    std::vector<float> pushConsts({ 2.0 });
+
+    auto algorithm = mgr.algorithm(params,
+                                   compileSource(shader),
+                                   workgroup,
+                                   specConsts,
+                                   pushConsts);
+
+    EXPECT_EQ(algorithm->getWorkgroup(), workgroup);
+    EXPECT_EQ(algorithm->getPushConstants<float>(), pushConsts);
+    EXPECT_EQ(algorithm->getSpecializationConstants<float>(), specConsts);
+}
